<?php

/**
 * @file
 * Additional functions.
 */

use Mailjet\Resources;
use Mailjet\Client;

/**
 * Return mailjet_api_statistics library.
 */
function mailjet_api_statistics_get_library() {
  $library_path = libraries_get_path(MAILJET_API_STATISTICS_LIBRARIES_PATH);
  require DRUPAL_ROOT . '/' . $library_path . MAILJET_API_STATISTICS_AUTOLOAD_PATH;
  libraries_load(MAILJET_API_STATISTICS_LIBRARY_NAME);
}

/**
 * Return mailjet_api client.
 */
function mailjet_api_statistics_get_mailjet_client() {
  $library_path = libraries_get_path(MAILJET_API_STATISTICS_LIBRARIES_PATH);
  require DRUPAL_ROOT . '/' . $library_path . MAILJET_API_STATISTICS_AUTOLOAD_PATH;
  libraries_load(MAILJET_API_STATISTICS_LIBRARY_NAME);

  $apikey = variable_get('mailjet_api_statistics_api_key', '');
  $apisecret = variable_get('mailjet_api_statistics_api_secret_key', '');
  $mj = new Client($apikey, $apisecret);

  return $mj;
}

/**
 * Send mail to mailjet and return response.
 */
function mailjet_api_statistics_basic_email($from_email, $from_name, $subject, $text_part, $html_part, $recipients) {
  $mj = mailjet_api_statistics_get_mailjet_client();
  $body = [
    'FromEmail' => $from_email,
    'FromName' => $from_name,
    'Subject' => $subject,
    'Text-part' => $text_part,
    'Html-part' => $html_part,
    'Recipients' => $recipients,
  ];

  $response = $mj->post(Resources::$Email, ['body' => $body]);

  if ($response->success()) {
    $message = t('The email sended. message_id = @message_id.',
      array(
        '@message_id' => $response->getData()['Sent'][0]['MessageID'],
      )
    );
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Sending basic email: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }

  return $response;
}

/**
 * Send mail to mailjet and return response.
 */
function mailjet_api_statistics_full_email($params) {
  $mj = mailjet_api_statistics_get_mailjet_client();
  // Build the Mailjet message array.
  $body = array(
    'FromEmail' => $params['from_email'],
    'FromName' => $params['from_name'],
    'Subject' => $params['subject'],
    'Text-part' => $params['text_part'],
    'Html-part' => $params['html_part'],
    'Recipients' => $params['recipients'],
  );

  if (!empty($params['headers'])) {
    $body['Headers'] = $params['headers'];
  }
  if (!empty($params['attachments'])) {
    $attachments = array();
    foreach ($params['attachments'] as $attachment) {
      if (file_exists($attachment)) {
        $attachments[] = [
          'Content-type' => $params['attachments']['filemime'],
          'Filename' => $params['attachments']['filename'],
          'content' => base64_encode(file_get_contents($params['attachments']['filepath'])),
        ];
      }
    }

    if (count($attachments) > 0) {
      $body['Attachments'] = $attachments;
    }
  }

  $response = $mj->post(Resources::$Email, ['body' => $body]);

  if ($response->success()) {
    $message = t('The email sended. message_id = @message_id.',
      array(
        '@message_id' => $response->getData()['Sent'][0]['MessageID'],
      )
    );
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Sending full email: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }

  return $response;
}

/**
 * Details of a specific Message (e-mail) processed by Mailjet.
 */
function mailjet_api_statistics_get_basic_information_about_message($message_id) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Message, ['id' => $message_id]);
  if ($response->success()) {
    $message = t('Get basic information about message. message_id = @message_id.',
      array(
        '@message_id' => $message_id,
      )
    );
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Get basic information about message: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Provides information for a specific message.
 */
function mailjet_api_statistics_get_complement_information_about_message($message_id) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Messageinformation, ['id' => $message_id]);
  if ($response->success()) {
    $message = t('Get complement information about message. message_id = @message_id.',
      array(
        '@message_id' => $message_id,
      )
    );
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Get complement information about message: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Event history of a message.
 */
function mailjet_api_statistics_get_message_history($message_id) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Messagehistory, ['id' => $message_id]);
  if ($response->success()) {
    $message = t('Get message history. message_id = @message_id.',
      array(
        '@message_id' => $message_id,
      )
    );
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Get message history: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Statuses and events summary for a specific message.
 */
function mailjet_api_statistics_get_messages_sent_statistics($message_id) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Messagesentstatistics, ['id' => $message_id]);
  if ($response->success()) {
    $message = t('Get messages sent statistics. message_id = @message_id.',
      array(
        '@message_id' => $message_id,
      )
    );
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Get messages sent statistics: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Get details of Messages in a campaign.
 */
function mailjet_api_statistics_get_details_about_campaign($campaign_id, $filters = array()) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $default_filters = array(
    'Campaign' => $campaign_id,
  );
  $filters = array_merge($default_filters, $filters);

  $response = $mj->get(Resources::$Message, ['filters' => $filters]);
  if ($response->success()) {
    $message = t('Get details of Messages in a campaign.');
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Get details of Messages in a campaign: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Get API key Campaign/Message statistics.
 */
function mailjet_api_statistics_get_messages_statistics($filters = array()) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  if (!empty($filters)) {
    $response = $mj->get(Resources::$Messagestatistics, ['filters' => $filters]);
  }
  else {
    $response = $mj->get(Resources::$Messagestatistics);
  }

  if ($response->success()) {
    $message = t('Get API key Campaign/Message statistics.');
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Get API key Campaign/Message statistics: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Retrieve information about messages opened at least once by their recipients.
 */
function mailjet_api_statistics_get_open_information($filters = array()) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Openinformation, ['filters' => $filters]);

  if ($response->success()) {
    $message = t('Retrieve informations about messages opened at least once by their recipients.');
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Retrieve informations about messages opened at least once by their recipients: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Get click statistics for messages.
 */
function mailjet_api_statistics_get_click_statistics($filters = array()) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Clickstatistics, ['filters' => $filters]);

  if ($response->success()) {
    $message = t('Get click statistics for messages.');
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Get click statistics for messages: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Get statistics on the bounces generated by emails sent on a given API Key.
 */
function mailjet_api_statistics_get_bounce_statistics($filters = array()) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Bouncestatistics, ['filters' => $filters]);

  if ($response->success()) {
    $message = t('Get statistics on the bounces generated by emails sent on a given API Key.');
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Get statistics on the bounces generated by emails sent on a given API Key: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Retrieve statistics on e-mails opened at least once by their recipients.
 */
function mailjet_api_statistics_get_open_statistics($filters = array()) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Openstatistics, ['filters' => $filters]);

  if ($response->success()) {
    $message = t('Retrieve statistics on e-mails opened at least once by their recipients.');
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Retrieve statistics on e-mails opened at least once by their recipients: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}

/**
 * Top links clicked historgram.
 */
function mailjet_api_statistics_get_top_link_clicked_histogram($filters = array()) {
  $result = FALSE;

  $mj = mailjet_api_statistics_get_mailjet_client();
  $response = $mj->get(Resources::$Toplinkclicked, ['filters' => $filters]);

  if ($response->success()) {
    $message = t('Top links clicked historgram.');
    $result = $response->getData();
  }
  else {
    $message = t('Something goes wrong! Reason = @reason',
      array(
        '@reason' => $response->getReasonPhrase(),
      )
    );
  }
  watchdog('mailjet_api_statistics', $message);

  if (variable_get('mailjet_api_statistics_enable_debug')) {
    $message = t('Top links clicked historgram: @response',
      array('@response' => var_export($response, TRUE)));
    watchdog('mailjet_api_statistics_debug', $message);
  }
  return $result;
}
